package idi.alexanbj.em;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;
import com.aliasi.classify.TradNaiveBayesClassifier;
import com.aliasi.corpus.ListCorpus;
import com.aliasi.io.Reporter;
import com.aliasi.lingmed.dao.DaoException;
import com.aliasi.lingmed.dao.SearchResults;
import com.aliasi.lingmed.medline.MedlineCodec;
import com.aliasi.lingmed.medline.parser.MedlineCitation;
import com.aliasi.tokenizer.EnglishStopTokenizerFactory;
import com.aliasi.tokenizer.LowerCaseTokenizerFactory;
import com.aliasi.tokenizer.PorterStemmerTokenizerFactory;
import com.aliasi.tokenizer.RegExTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.util.Factory;

public class ExpectationMaximization {
	
	static final int MAX_ITER = 25;
	static final double THRESHOLD = 0.0001;
	static final double TOKEN_COUNT = 0.0001;
	static final double DOC_LENGTH_NORMALIZER = 9.0;
	static final double CATEGORY_PRIOR = 0.005;
	static final double TOKEN_IN_CATEGORY_PRIOR = 0.001;
	static final double INITIAL_TOKEN_IN_CATEGORY_PRIOR = 0.1;
	
	static final TokenizerFactory TOKENIZER_FACTORY = tokenizerFactory();
	
	public static TradNaiveBayesClassifier em(Topic topic, SearchResults<MedlineCitation> results, Reporter reporter) throws DaoException {
		
		System.out.println("CATEGORY PRIOR=");
		System.out.println("TOKEN IN CATEGORY PRIOR=");
		
		System.out.println("MAX ITERATIONS=" + MAX_ITER);
		
		
		ListCorpus corpus = new ListCorpus<Classified<String>>();
		ListCorpus unlabeledCorpus = new ListCorpus<String>();

		Classification relevant = new Classification("Relevant");
		Classification irrelevant = new Classification("Irrelevant");
		final Set<String> categorySet = new HashSet<String>() {{
			add("Relevant");
			add("Irrelevant");
		}};
		
		// just for early instantiation
		MedlineQRELS.getInstance();
		
		int trainedRelevant = 0;
		int trainedIrrelevant = 0;
		
		// set i to result set size or max 1000 to prevent indexoutofboundsexception
		for (int i = 0; i < (results.size() < 1000 ? results.size() : 1000); ++i) {
			MedlineCitation ml = results.getResult(i);
			boolean relevance = MedlineQRELS.getInstance().relevant(ml.pmid(), topic.getID());		
			if (relevance && trainedRelevant < 3) {
				Classified<String> classified;
				classified = new Classified<String>(MedlineCodec.titleAbstract(ml), relevant);
				corpus.addTrain(classified);
				++trainedRelevant;
			} else if (!relevance && trainedIrrelevant < 3) {
				Classified<String> classified;
				classified = new Classified<String>(MedlineCodec.titleAbstract(ml), irrelevant);
				//System.out.println(classified.getObject());
				corpus.addTrain(classified);
				++trainedIrrelevant;
			} else {
				unlabeledCorpus.addTrain(MedlineCodec.titleAbstract(ml));
			}	
		}
		
		
		System.out.println("Supervised training set size: " + corpus.trainCases().size());
		System.out.println("Relevant trained: " + trainedRelevant);
		System.out.println("Irrelevant trained: " + trainedIrrelevant);
		TradNaiveBayesClassifier initialClassifier = new TradNaiveBayesClassifier(categorySet, TOKENIZER_FACTORY, CATEGORY_PRIOR, INITIAL_TOKEN_IN_CATEGORY_PRIOR, DOC_LENGTH_NORMALIZER);
		
		Factory<TradNaiveBayesClassifier> classifierFactory = new Factory<TradNaiveBayesClassifier>() {
			public TradNaiveBayesClassifier create() {
				return new TradNaiveBayesClassifier(categorySet, TOKENIZER_FACTORY, CATEGORY_PRIOR, TOKEN_IN_CATEGORY_PRIOR, DOC_LENGTH_NORMALIZER);
			}
		};
		
		
		TradNaiveBayesClassifier emClassifier = null;
		
		try {
			emClassifier = TradNaiveBayesClassifier.emTrain(initialClassifier, classifierFactory, corpus, unlabeledCorpus, TOKEN_COUNT, MAX_ITER, THRESHOLD, reporter);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return emClassifier;
		
		//reporter.close();
	}
	
	/**
	 * Setup the same analyzer as when building the Lucene index of Medline Citation entries.
	 */
	static TokenizerFactory tokenizerFactory() {
		TokenizerFactory factory = new RegExTokenizerFactory("\\p{L}+|\\p{Digit}+");
		factory = new LowerCaseTokenizerFactory(factory);
		factory = new EnglishStopTokenizerFactory(factory);
		factory = new PorterStemmerTokenizerFactory(factory);
		return factory;
	}
}
