package idi.alexanbj.em;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;
import com.aliasi.corpus.Corpus;
import com.aliasi.corpus.ObjectHandler;
import com.aliasi.lingmed.medline.parser.MedlineCitation;


public class MedlineCorpus extends Corpus<ObjectHandler<Classified<CharSequence>>> {
	
	private Map<String,String[]> mTrainingCatToTexts;
	private Map<String,String[]> mTestCatToTexts;
	private int mMaxSupervisedInstancesPerCategory = 1;
	
	public MedlineCorpus(String path) {
		
		Map<String,String[]> mTrainingCatToTexts = new HashMap<String,String[]>();
		Map<String,String[]> mTestCatToTexts = new HashMap<String,String[]>();

	}
	
	/**
	 * Returns set of categories for the corpus.
	 * 
	 * @return The set of categories for the corpus.
	 */
	public Set<String> categorySet() {
		return Collections.unmodifiableSet(mTrainingCatToTexts.keySet());
	}
	
	public void setMaxSupervisedInstancesPerCategory(int max) {
		mMaxSupervisedInstancesPerCategory = max;
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		
		int totalTrain = 0;
		int totalTest = 0;
		
		sb.append("TOTALS: #train=" + totalTrain
				+ " #test=" + totalTest
				+ " #combined=" + (totalTrain + totalTest));
		sb.append('\n');
		return sb.toString();
	}
	
	
	@Override
	public void visitTrain(ObjectHandler<Classified<CharSequence>> handler) {
		visit(mTrainingCatToTexts, handler, mMaxSupervisedInstancesPerCategory);
	}
	
	@Override
	public void visitTest(ObjectHandler<Classified<CharSequence>> handler) {
		//visit()
		throw new UnsupportedOperationException();
	}
	
	/**
	 * The same as for the labeled corpus, but here we forego the categories
	 * @return
	 */
	public Corpus<ObjectHandler<CharSequence>> unlabeledCorpus() {
		return new Corpus<ObjectHandler<CharSequence>> () {
			public void visitTest(ObjectHandler<CharSequence> handler) {
				throw new UnsupportedOperationException();
			}
			public void visitTrain(ObjectHandler<CharSequence> handler) {
				for (String[] texts : mTrainingCatToTexts.values()) {
					for (int i = 0; i < texts.length; i++) {
						handler.handle(texts[i]);
					}
				}
			}
		};
	}
	
	public void addDocument(MedlineCitation citation) {
		
		
		System.out.println(citation.pmid());
		
	}
	
	/**
	 * Helper method for convenience.
	 * @param catToItems
	 * @param handler
	 * @param maxItems
	 */
	private void visit(Map<String,String[]> catToItems, ObjectHandler<Classified<CharSequence>> handler, int maxItems) {
		for (Map.Entry<String,String[]> entry: catToItems.entrySet()) {
			String cat = entry.getKey();
			
			Classification c = new Classification(cat);
			String[] texts = entry.getValue();
			
			for (int i = 0; i < maxItems && i < texts.length; ++i) {
				Classified<CharSequence> classifiedText = new Classified<CharSequence>(texts[i],c);
				handler.handle(classifiedText);
			}
		}
	}
}
