package idi.alexanbj.em;

import java.io.File;
import java.io.IOException;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.store.FSDirectory;
import org.xml.sax.InputSource;

import com.aliasi.lingmed.medline.MedlineCodec;
import com.aliasi.lingmed.medline.SearchableMedlineCodec;
import com.aliasi.lingmed.medline.parser.MedlineCitation;
import com.aliasi.lingmed.medline.parser.MedlineParser;
import com.aliasi.lingmed.utils.FileUtils;

public class MedlineIndexer {
	
	private IndexWriter mIndexWriter;
	private MedlineCodec mMedlineCodec;
	private File mIndex;
	private String mXMLFilepath;
	
	private final static double RAM_BUF_SIZE = 1000d;
	private final static int MERGE_FACTOR_HI = 100;
	
	public MedlineIndexer(String xmlFilePath, String indexFilePath) throws IOException {
		
		mMedlineCodec = new SearchableMedlineCodec();
		mXMLFilepath = xmlFilePath;
		
		mIndex = FileUtils.checkIndex(indexFilePath, true);
		
	}
	
	public void run() {
		try {
			mIndexWriter = new IndexWriter(FSDirectory.getDirectory(mIndex), mMedlineCodec.getAnalyzer(), new IndexWriter.MaxFieldLength(IndexWriter.DEFAULT_MAX_FIELD_LENGTH));
		
			//mIndexWriter.setRAMBufferSizeMB(RAM_BUF_SIZE);
			//mIndexWriter.setMergeFactor(MERGE_FACTOR_HI);
			
			MedlineParser parser = new MedlineParser(true);
			
			MyMedlineHandler handler = new MyMedlineHandler(this);
			
			parser.setHandler(handler);
			
			System.out.println("Processing file: " + mXMLFilepath);
			
			InputSource inputSource = new InputSource(mXMLFilepath);
			parser.parse(inputSource);
			
			System.out.println("Started optimizing index.");
			mIndexWriter.optimize();
			mIndexWriter.close();
			
			System.out.println("Processing complete.");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	void addDocument(MedlineCitation citation) {

		Document doc = mMedlineCodec.toDocument(citation);
		
		try {
			mIndexWriter.addDocument(doc);
		} catch (CorruptIndexException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	public static void main(String[] args) {
		MedlineIndexer indexer;
		try {
			indexer = new MedlineIndexer("/home/alexanbj/MEDLINE/2004_TREC_XML_MEDLINE", "/home/alexanbj/MEDLINE/index");
			indexer.run();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

}
