package idi.alexanbj.em;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;

public class MedlineQRELS {
	
	private static MedlineQRELS mInstance = null;
	private Map<String,Set<String>> mTopicMap;
	
	public static MedlineQRELS getInstance() {
		if (mInstance == null) {
			try {
				mInstance = new MedlineQRELS();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return mInstance;
	}
	
	private MedlineQRELS() throws FileNotFoundException {
		
		mTopicMap = new HashMap<String,Set<String>>();
		Scanner scanner = new Scanner(new FileReader("/home/alexanbj/MEDLINE/04.qrels.txt"));
		
		try {
			while ( scanner.hasNextLine() ) {
				processLine( scanner.nextLine() );
			}
		} finally {
			scanner.close();
		}
	}
	
	public boolean relevant(String pmid, String topic) {
		
		Set<String> set = mTopicMap.get(pmid);
	
		return set != null ? set.contains(topic) : false;
	}
	
	private void processLine(String nextLine) {
		Scanner scanner = new Scanner(nextLine);
		scanner.useDelimiter("\t");
		if (scanner.hasNext() ) {
			String topic = scanner.next();
			scanner.next(); //skip iteration
			String pmid = scanner.next();
			scanner.next(); //skip relevance judgments
			
			Set<String> set = mTopicMap.get(pmid);

			if (set == null) {
				set = new HashSet<String>();
				mTopicMap.put(pmid, set);
			}
			set.add(topic);
			
		} else {
			throw new Error("Unable to process qrels line");
		}
	}
}
