package idi.alexanbj.em;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import com.aliasi.classify.TradNaiveBayesClassifier;
import com.aliasi.io.LogLevel;
import com.aliasi.io.Reporter;
import com.aliasi.io.Reporters;
import com.aliasi.lingmed.dao.DaoException;
import com.aliasi.lingmed.dao.SearchResults;
import com.aliasi.lingmed.medline.MedlineCodec;
import com.aliasi.lingmed.medline.MedlineSearcher;
import com.aliasi.lingmed.medline.MedlineSearcherImpl;
import com.aliasi.lingmed.medline.SearchableMedlineCodec;
import com.aliasi.lingmed.medline.parser.MedlineCitation;
import com.aliasi.util.Strings;

public class Search {
	
	public static void main(String[] args) throws IOException, DaoException {
		MedlineCodec medlineCodec = new SearchableMedlineCodec();
		
		Directory index = FSDirectory.getDirectory("/home/alexanbj/MEDLINE/index");
		
		IndexSearcher isearcher = new IndexSearcher(index, true); // read only
		MedlineSearcher medlineSearcher = new MedlineSearcherImpl(medlineCodec, isearcher);
		
		Reporter reporter = Reporters.stream(System.out, "UTF-8").setLevel(LogLevel.DEBUG);
		
		Date date = Calendar.getInstance().getTime();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd-HH.mm");
		String timestamp = sdf.format(date);
		
		
		FileWriter fstream = new FileWriter("/home/alexanbj/MEDLINE/runs/"+ timestamp + "-ir.txt");
		BufferedWriter irOut = new BufferedWriter(fstream);
		fstream = new FileWriter("/home/alexanbj/MEDLINE/runs/"+ timestamp + "-clustered.txt");
		BufferedWriter clusteredOut = new BufferedWriter(fstream);
		
		for (Topic t : Topic.getTopics()) {
			// Search for title in title and abstract, need in abstract
	
			String query = SearchableMedlineCodec.TITLE_FIELD + ":(" + t.getTitle() + ")^2 " +
				SearchableMedlineCodec.ABSTRACT_FIELD + ":(" + t.getTitle() + ")"; // +
				/*SearchableMedlineCodec.TITLE_FIELD + ":(" + t.getContext() + ") " + 
				SearchableMedlineCodec.ABSTRACT_FIELD + ":(" + t.getContext() + ") " + 
				SearchableMedlineCodec.ABSTRACT_FIELD + ":(" + t.getTitle() + ") " + 
				SearchableMedlineCodec.ABSTRACT_FIELD + ":(" + t.getNeed() + ")";*/
			
			System.out.println(query);
			SearchResults<MedlineCitation> results = medlineSearcher.search(query);

			System.out.println("Topic: " + t.getID() + " Results: " + results.size() + " Title: " + t.getTitle());
			TradNaiveBayesClassifier emClassifier = ExpectationMaximization.em(t, results, reporter);
			
			int relevantCount = 0;
			// set i to result set size or max 1000 to prevent indexoutofboundsexception
			for (int i = 0; i < (results.size() < 1000 ? results.size() : 1000); ++i) {
				MedlineCitation ml = results.getResult(i);
				
				irOut.write(t.getID() + " Q0 " + ml.pmid() + "\t" + (i+1) + "\t" + results.getScore(i) + "\ttag1\n");
	
				if (emClassifier.classify(MedlineCodec.titleAbstract(ml)).bestCategory() == "Relevant") {
					++relevantCount;
					clusteredOut.write(t.getID() + " Q0 " + ml.pmid() + "\t" + relevantCount + "\t" + results.getScore(i) + "\ttag1\n");
				}
			}
			System.out.println("Number of articles classified as relevant: " + relevantCount);
		}
		
		reporter.close();
		irOut.close();
		clusteredOut.close();
	}
}
